/*------------------------------------------------------------
 *                              CACTI 6.5
 *         Copyright 2008 Hewlett-Packard Development Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein, and
 * hereby grant back to Hewlett-Packard Company and its affiliated companies ("HP")
 * a non-exclusive, unrestricted, royalty-free right and license under any changes,
 * enhancements or extensions  made to the core functions of the software, including
 * but not limited to those affording compatibility with other hardware or software
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to HP any such changes,
 * enhancements or extensions that they make and inform HP of noteworthy uses of
 * this software.  Correspondence should be provided to HP at:
 *
 *                       Director of Intellectual Property Licensing
 *                       Office of Strategy and Technology
 *                       Hewlett-Packard Company
 *                       1501 Page Mill Road
 *                       Palo Alto, California  94304
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND HP DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL HP
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *------------------------------------------------------------*/


#ifndef __CROSSBAR__
#define __CROSSBAR__

#include <assert.h>
#include <iostream>
#include "basic_circuit.h"
#include "cacti_interface.h"
#include "component.h"
#include "parameter.h"
#include "mat.h"
#include "wire.h"

class Crossbar : public Component
{
  public:
    Crossbar(
      double in,
      double out,
      double flit_sz,
      TechnologyParameter::DeviceType *dt = &(g_tp.peri_global));
    ~Crossbar();

    void print_crossbar();
    double output_buffer();
    void compute_power();

    double n_inp, n_out;
    double flit_size;
    double tri_inp_cap, tri_out_cap, tri_ctr_cap, tri_int_cap;

  private:
	  double CB_ADJ;
	  /*
	   * Adjust factor of the height of the cross-point (tri-state buffer) cell (layout) in crossbar
	   * buffer is adjusted to get an aspect ratio of whole cross bar close to one;
	   * when adjust the ratio, the number of wires route over the tri-state buffers does not change,
	   * however, the effective wiring pitch changes. Specifically, since CB_ADJ will increase
	   * during the adjust, the tri-state buffer will become taller and thiner, and the effective wiring pitch
	   * will increase. As a result, the height of the crossbar (area.h) will increase.
	   */

	TechnologyParameter::DeviceType *deviceType;
    double TriS1, TriS2;
    double min_w_pmos, Vdd;

};




#endif
